import React, { useState } from 'react'
import { useDrop } from 'react-dnd'
import update from 'immutability-helper'
import { is, fromJS } from 'immutable'
import { Col } from 'antd'
import Utils from '@/utils/utils.js'
import Card from './card'
import ItemTypes from './itemtypes'
import './index.scss'

const Container = ({list, type, groupId, setting, handleList, handleMenu, deleteMenu, profileMenu }) => {
  const [cards, setCards] = useState(list)
  const moveCard = (id, atIndex) => {
    const { card, index } = findCard(id)

    if (!card) return

    const _cards = update(cards, { $splice: [[index, 1], [atIndex, 0, card]] })
    if (type === 'action') {
      handleList(type, _cards)
    } else {
      handleList(type, _cards, null, groupId)
    }
  }

  if (!is(fromJS(cards), fromJS(list))) {
    setCards(list)
  }

  const findCard = id => {
    const card = cards.filter(c => `${c.uuid}` === id)[0]
    return {
      card,
      index: cards.indexOf(card),
    }
  }

  const editCard = id => {
    const { card } = findCard(id)
    handleMenu(card)
  }

  const copyCard = id => {
    const { card } = findCard(id)

    let _card = JSON.parse(JSON.stringify(card))

    _card.originUuid = _card.uuid
    _card.uuid = Utils.getuuid()
    _card.iscopy = true
    _card.focus = true

    // 复制到剪切板
    let oInput = document.createElement('input')
    let val = JSON.parse(JSON.stringify(_card))
    val.copyType = 'form'
    val.uuid = Utils.getuuid()

    delete val.originUuid
    delete val.iscopy

    delete val.$srcId
    
    let srcid = localStorage.getItem(window.location.href.split('#')[0] + 'srcId')
    if (srcid) {
      val.$srcId = srcid
    }

    oInput.value = window.btoa(window.encodeURIComponent(JSON.stringify(val)))
    document.body.appendChild(oInput)
    oInput.select()
    document.execCommand('Copy')
    oInput.className = 'oInput'
    oInput.style.display = 'none'
    document.body.removeChild(oInput)

    handleMenu(_card)
  }
  
  const profileCard = id => {
    const { card } = findCard(id)
    profileMenu(card)
  }

  const delCard = id => {
    const { card } = findCard(id)
    deleteMenu({card: card, type: type})
  }

  const [, drop] = useDrop({
    accept: ItemTypes[type],
    drop(item) {
      if (item.hasOwnProperty('originalIndex') && groupId) {
        const { card } = findCard(item.id)

        if (!card) {
          handleList(type, cards, null, groupId, item.id)
        }
      }

      if (item.hasOwnProperty('originalIndex')) {
        return
      }

      let newcard = {}
      newcard.uuid = Utils.getuuid()
      newcard.focus = true
      
      if (item.type === 'search') {
        let _match = 'like'
        if (item.subType === 'select' || item.subType === 'link') {
          _match = '='
        } else if (item.subType === 'date' || item.subType === 'datemonth') {
          _match = '>='
        } else if (item.subType === 'dateweek' || item.subType === 'daterange') {
          _match = 'between'
        }
        
        newcard.label = 'label'
        newcard.field = ''
        newcard.initval = ''
        newcard.type = item.subType
        newcard.resourceType = '0'
        newcard.options = []
        newcard.orderType = 'asc'
        newcard.match = _match
      } else if (item.type === 'action') {
        newcard.label = 'button'
        newcard.Ot = 'requiredSgl'
        newcard.OpenType = item.subType
        newcard.tabType = 'SubTable'
        newcard.linkTab = ''
        newcard.class = 'default'
        newcard.intertype = 'inner'
        newcard.position = 'toolbar'
        newcard.execSuccess = 'grid'
        newcard.execError = 'never'
        newcard.popClose = 'never'
        newcard.verify = null
      }
      
      let targetId = ''

      if (item.dropTargetId) {
        targetId = item.dropTargetId
        delete item.dropTargetId
      } else if (cards.length > 0) {
        targetId = cards.slice(-1)[0].uuid
      }

      const { index: overIndex } = findCard(`${targetId}`)
      const _cards = update(cards, { $splice: [[overIndex + 1, 0, newcard]] })

      handleList(type, _cards, newcard, groupId)
    }
  })

  let _width = (setting && setting.width) || 100
  
  return (
    <div ref={drop} className="ant-row" style={type === 'search' ? {margin: '0 auto', width: _width + '%'} : {}}>
      {type === 'action' && cards.map(card => (
        <Card
          key={card.uuid}
          id={card.uuid}
          type={type}
          card={card}
          moveCard={moveCard}
          editCard={editCard}
          delCard={delCard}
          profileCard={profileCard}
          findCard={findCard}
        />
      ))}
      {type === 'search' && cards.map(card => (
        <Col key={card.uuid} span={card.type !== 'textarea' ? 24 / setting.cols : 24}>
          <Card
            id={card.uuid}
            key={card.uuid}
            type={type}
            card={card}
            cols={setting.cols}
            moveCard={moveCard}
            editCard={editCard}
            delCard={delCard}
            copyCard={copyCard}
            findCard={findCard}
          />
        </Col>
      ))}
      {cards.length === 0 &&
        <div className="common-drawarea-placeholder">
          {type === 'action' ? '请添加按钮' : '请添加表单'}
        </div>
      }
    </div>
  )
}
export default Container
